function [retval1, retval2, retval3,retval4]=dekompozicijaPetdijagonalne(F)
% DSYINF: Decomposition F = G * JJ * G', where F is a non-zero
%         real five-diagonal symmetric matrix, JJ = diag(+-1), 
%         and G has full column rank.
%         On exit vector P contains the pivoting which took place.
% Usage: [G,JJ,P,err]=dekompozicijaPetdijagonalne(F)
% Standard precision (double).
% Uses ALGORITHM E for diagonal pivoting.
% For details see: 
% [1] I. Slapnicar: Analysis of real symmetric and Hermitian
%    decomposition, Tech. report, University of Split, 1995.
%    (ivan.slapnicar@fesb.hr)             
% [2] J.R. Bunch and L.Kaufman: Some stable methods for calculating inertia
%    and solving symmetric linear systems, Mathmematics of computation,
%    vol.31, nb.137, 1977, pp 163-179
%%%  Make the current matrix symmetric
F=1/2*(F+F');
FF=F;
[n,m]=size(F);
%%%  Initialize alpha, npos and r, and the vectors J and P.   
%alpha = ( 1 + sqrt(17) )/8;
alpha=0.52542;
i = 1;
npos = 0;
r = n;
P = 1:n;
JJ=1:n;
%%%  Main loop.
while i <= n-2,
%%%  Make the current submatrix symmetric.
     in=i:n; % indices of working array
  %  if (n-i)>0; F(in,in) = tril(F(in,in)) + tril(F(in,in),1)'; end;

  
   if abs(F(i+1,i)) >= abs(F(i+2,i)), % STEP (1)
       if n-i==2,
           sigma=max(abs([F(i+1,i),F(i+2,i+1)]));
       elseif n-i>2,
          sigma=max(abs([F(i+1,i),F(i+2,i+1),F(i+3,i+1)]));
       end
       
       if  sigma*abs(F(i,i))>=alpha*(abs(F(i+1,i)))^2,   % STEP (1) a)
%%%  1 x 1 pivot.
%%%  Update F.
          JJ(i) = sign(F(i,i));
          npos = npos + (1 + JJ(i)) / 2;
          F(i,i) = sqrt(abs(F(i,i)));
          if i < n,
            i1n=i+1:n;
            F(i1n,i) = F(i1n,i) * ( JJ(i) / F(i,i) );
            F(i,i1n) = zeros(1,n-i);
            F(i1n,i1n) = F(i1n,i1n) - JJ(i) * F(i1n,i) * ...
                             F(i1n,i)';
          end;
          i = i + 1;
       elseif abs(F(i+1,i+1))>=sigma, % STEP (1) b)
%%% interchange the first and second rows and columns
%%% and notify the permutation in vector P
          s=i+1;
          F([i s],:) = F([s i],:);
          F(in,[i s]) = F(in,[s i]);
          P([i s]) = P([s i]);
%%%  1 x 1 pivot.
%%%  Update F.
          JJ(i) = sign(F(i,i));
          npos = npos + (1 + JJ(i)) / 2;
          F(i,i) = sqrt(abs(F(i,i)));
          if i < n,
            i1n=i+1:n;
            F(i1n,i) = F(i1n,i) * ( JJ(i) / F(i,i) );
            F(i,i1n) = zeros(1,n-i);
            F(i1n,i1n) = F(i1n,i1n) - JJ(i) * F(i1n,i) * ...
                             F(i1n,i)';
          end;
          i = i + 1;
       else % STEP (1) c)
%%%  2 x 2 pivot.
%%%  Compute the 2 x 2 orthogonal matrix.
          z = ( F(i+1,i+1) - F(i,i) ) / ( 2 * F(i+1,i) );
          signz=sign(z); if signz==0; signz=1;end;
          t = signz / ( abs(z) + sqrt( z^2 + 1 ) );
          h = sqrt( 1 + t^2 );
          cs = 1 / h;
          sn = t / h;
          Q = [cs sn; -sn cs];
%%%  Updating F.
          a = F(i,i) - F(i+1,i) * t;
          b = F(i+1,i+1) + F(i+1,i) * t;
          JJ(i:i+1) = sign([a b]);
          npos = npos + 1;
          D = sqrt(diag(abs([a b])));
          F(i:i+1,i:i+1) = Q * D;
           if i < n-1,
            i2n=i+2:n;
            F(i2n,i:i+1) = F(i2n,i:i+1) * (Q * diag(JJ(i:i+1)) * inv(D));
            F(i:i+1,i2n) = zeros(2,n-i-1);
            F(i2n,i2n) = F(i2n,i2n) - F(i2n,i:i+1) * ... 
                          diag(JJ(i:i+1)) * F(i2n,i:i+1)';
          end;
          i = i + 2;
       end; % OF STEP (1) a) or b) or c)
   else % abs(F(i+1,i)) < abs(F(i+2,i)), % STEP (2)
       if n-i==3,
          sigma=max(abs(F(i+1:i+3,i+2)));
       elseif n-i==2,
           sigma=max(abs(F(i+1:i+2,i+2)));
       elseif n-i>2,
           sigma=max(abs(F(i+1:i+4,i+2)));
       end;
       if sigma*abs(F(i,i))>=alpha*(abs(F(i+2,i)))^2% STEP (2) a)
%%%  1 x 1 pivot.
%%%  Update F.
          JJ(i) = sign(F(i,i));
          npos = npos + (1 + JJ(i)) / 2;
          F(i,i) = sqrt(abs(F(i,i)));
          if i < n,
            i1n=i+1:n;
            F(i1n,i) = F(i1n,i) * ( JJ(i) / F(i,i) );
            F(i,i1n) = zeros(1,n-i);
            F(i1n,i1n) = F(i1n,i1n) - JJ(i) * F(i1n,i) * ...
                             F(i1n,i)';
          end;
          i = i + 1;
       else % STEP (2) b)
%%% Interchange the second and third rows and columns of F.
%%% Bring the elements F(p,p) and F(q,q) to the
%%%  positions F(i+1,i+1) and F(i,i), respectively, and notify the
%%%  permutation in vector P.
          p=i+1;q=i+2;
          F([i q],:) = F([q i],:);
          F([i+1 p],:) = F([p i+1],:);
          F(in,[i q]) = F(in,[q i]);
          F(in,[i+1 p]) = F(in,[p i+1]);
          P([i q]) = P([q i]);
          P([i+1 p]) = P([p i+1]);
%%%  2 x 2 pivot.
%%%  Compute the 2 x 2 orthogonal matrix.
          z = ( F(i+1,i+1) - F(i,i) ) / ( 2 * F(i+1,i) );
          signz=sign(z); if signz==0; signz=1;end;
          t = signz / ( abs(z) + sqrt( z^2 + 1 ) );
          h = sqrt( 1 + t^2 );
          cs = 1 / h;
          sn = t / h;
          Q = [cs sn; -sn cs];
%%%  Updating F.
          a = F(i,i) - F(i+1,i) * t;
          b = F(i+1,i+1) + F(i+1,i) * t;
          JJ(i:i+1) = sign([a b]);
          npos = npos + 1;
          D = sqrt(diag(abs([a b])));
          F(i:i+1,i:i+1) = Q * D;
           if i < n-1,
            i2n=i+2:n;
            F(i2n,i:i+1) = F(i2n,i:i+1) * (Q * diag(JJ(i:i+1)) * inv(D));
            F(i:i+1,i2n) = zeros(2,n-i-1);
            F(i2n,i2n) = F(i2n,i2n) - F(i2n,i:i+1) * ... 
                          diag(JJ(i:i+1)) * F(i2n,i:i+1)';
          end;
         i = i + 2;
       end; % OF STEP (2) a) or b)
   end; % OF STEP (2)

end; % OF WHILE (MAIN LOOP)


%%% last 2x2 submatrix or 1x1 submatrix: use complete diagonal pivoting
%%% (see Appendix in [2])
%%% now i==n-1 or i==n
alpha0=(1+sqrt(17))/8;

while i<=n, 
    in=i:n;
    [temp,p]=max(abs(F(in,in)));
    [mu0,q]=max(temp);
    p=p(q);
    [mu1,s]=max(abs(diag(F(in,in))));
    if mu1>=alpha0*mu0,
%%%  1 x 1 pivot. If the block currently being reduced is singular, 
%%%  the algorithm is finished.
      if mu1 == 0,
         r = i - 1;
         i = n + 1;
      else
%%%  Bring the element F(s,s) to the position (i,i), and notify the 
%%%  permutation in the vector P.
         if s==2,
            s = s + i - 1;
            F([i s],:) = F([s i],:);
            F(in,[i s]) = F(in,[s i]);
            P([i s]) = P([s i]);
         end;
%%%  Update F.
         JJ(i) = sign(F(i,i));
         npos = npos + (1 + JJ(i)) / 2;
         F(i,i) = sqrt(abs(F(i,i)));
         if i < n,
            i1n=i+1:n;
            F(i1n,i) = F(i1n,i) * ( JJ(i) / F(i,i) );
            F(i,i1n) = zeros(1,n-i);
            F(i1n,i1n) = F(i1n,i1n) - JJ(i) * F(i1n,i) * ...
                             F(i1n,i)';
         end;
         
         %%% last 1x1 pivot
         JJ(n)=sign(F(n,n));
         F(n,n)=sqrt(abs(F(n,n)));
         break
      end;
    else
%%%  2 x 2 pivot.
%%%  Compute the 2 x 2 orthogonal matrix.
          z = ( F(i+1,i+1) - F(i,i) ) / ( 2 * F(i+1,i) );
          signz=sign(z); if signz==0; signz=1;end;
          t = signz / ( abs(z) + sqrt( z^2 + 1 ) );
          h = sqrt( 1 + t^2 );
          cs = 1 / h;
          sn = t / h;
          Q = [cs sn; -sn cs];
%%%  Updating F.
          a = F(i,i) - F(i+1,i) * t;
          b = F(i+1,i+1) + F(i+1,i) * t;
          JJ(i:i+1) = sign([a b]);
          npos = npos + 1;
          D = sqrt(diag(abs([a b])));
          F(i:i+1,i:i+1) = Q * D;
           if i < n-1,
            i2n=i+2:n;
            F(i2n,i:i+1) = F(i2n,i:i+1) * (Q * diag(JJ(i:i+1)) * inv(D));
            F(i:i+1,i2n) = zeros(2,n-i-1);
            F(i2n,i2n) = F(i2n,i2n) - F(i2n,i:i+1) * ... 
                          diag(JJ(i:i+1)) * F(i2n,i:i+1)';
          end;         break
    end; % of mu1>=alpha0*mu0
end; % last 2x2 submatrix
   
 
%%%  If non-singularity did not occur, then rank = dimension.
if r == 0,
   r = n;
end;


%%% Permutiram stupce matrice G da JJ bude [I 0 ]
%%%                                        [0 -I]  
% [JJ,IX]=sort(JJ(1:r),'descend');
% F(P,1:r) = F(:,1:r);
% temp=F(:,1:r);
% G=zeros(r);
% for i=1:r
%     G(:,i)=temp(:,IX(i));
% end;
% retval1=G;
% retval2=diag(JJ(1:r));
% retval3=P;
% retval4=norm(retval1*retval2*retval1'-FF);

F(P,1:r) = F(:,1:r);
retval1=F(:,1:r);
retval2=diag(JJ(1:r));
retval3=P;
retval4=norm(retval1*retval2*retval1'-FF);
