% Test1: Use exact inverse of K, M as the preconditioner.

clear;
close all;

load KMsih4C;
n = size(K, 1);
m = 1;n_pos=1;
I = speye(n);
A = blkdiag(K, M);
B = [0*I, I; I, 0*I];

tol = 1e-7;
maxiter = 1000;
opts.nitn = maxiter;
opts.tol = tol;
flag = [1 1 1 1 1 1];

X = full(I(:, 1:m));
Y = X;
% Z = [X; Y];
% Xini = [X, X; Y, -Y];

if flag(1),
msch=2; 
fprintf('indef_m_scheme_KM with m=%d.\n and shift=0',msch);
tic;
tol=1e-7;opts.nitn=100;opts.met=3;
[D1, Z1, iter1, hist1]= indef_m_scheme_KM(K,M,n_pos,X,Y,tol,opts,msch);
t1=toc;
D1'
iter1
iter1 = size(h1.res, 1)-1;
fprintf('#(iterations) = %d.\n\n', iter1);
end

if flag(2),
msch=3; 
shift = 0.54;
opts.shift = shift;
fprintf('indef_m_scheme_KM with m=%d and shift=%2.3f\n',msch,shift);
tic;
tol=1e-7;opts.nitn=100;opts.met=6;
[D2, Z2, iter2, hist2]= indef_m_scheme_KM(K,M,n_pos,X,Y,tol,opts,msch);
t2=toc;
D2'
fprintf('#(iterations) = %d.\n\n', iter2);
end

if flag(3),
fprintf('General indefinite LOBPCG with one shift\n');
tic;
RK = chol(K);
RM = chol(M);
TK = @(x) RK\(RK'\x);
TM = @(x) RM\(RM'\x);
T = @(x) [TK(x(1:(size(x,1)/2), :)); TM(x((size(x,1)/2+1):end, :))];
[D3, X3, iter3, h3] = indef_lobpcg(A, B, T, [], Xini, m, m, tol, maxiter);
toc;
D3(m+1:end)'
fprintf('#(iterations) = %d.\n\n', iter3);
end

opts.met = 5;
shift = 0.54;
opts.shift = shift;

if flag(4),
fprintf('LOBP4dCG with a nonzero shift\n');
tic;
[D4, X4, h4] = LOBP4dCG(K, M, m, Z, opts);
toc;
D4'
iter4 = size(h4.res, 1)-1;
fprintf('#(iterations) = %d.\n\n', iter4);
end

if flag(5),
fprintf('Special indefinite LOBPCG with two shifts\n');
tic;
[D5, X5, iter5, h5] = ILOBPCG_KM(K, M, m, X, Y, tol, opts);
toc;
D5'
fprintf('#(iterations) = %d.\n\n', iter5);
end

if flag(6),
fprintf('General indefinite LOBPCG with two shifts\n');
tic;
RL = chol(A+shift*B);
RR = chol(A-shift*B);
TL = @(x) RL\(RL'\x);
TR = @(x) RR\(RR'\x);
[D6, X6, iter6, h6] = indef_lobpcg(A, B, TR, TL, Xini, m, m, tol, maxiter);
toc;
D6(m+1:end)'
fprintf('#(iterations) = %d.\n\n', iter6);
end

if min(flag) == 1,
figure(1);
semilogy(1:h1.deflate(1), abs(h1.ev(1:h1.deflate(1), 1)-Heig(1))/abs(Heig(1)), 'r');
hold on;
semilogy(1:h1.deflate(2), abs(h1.ev(1:h1.deflate(2), 2)-Heig(2))/abs(Heig(2)), 'b');
semilogy(1:h1.deflate(3), abs(h1.ev(1:h1.deflate(3), 3)-Heig(3))/abs(Heig(3)), 'm');
semilogy(1:h1.deflate(4), abs(h1.ev(1:h1.deflate(4), 4)-Heig(4))/abs(Heig(4)), 'k');
xlabel('iteration');
ylabel('error');
title('relative error in positive eigenvalues');
legend('Err(\lambda_1)', 'Err(\lambda_2)', 'Err(\lambda_3)', 'Err(\lambda_4)');
print('-depsc2', 'test1_1.eps');

figure(2);
semilogy(1:h2.deflate(1), abs(h2.eigen(1:h2.deflate(1), 1)-Heig(1))/abs(Heig(1)), 'r');
hold on;
semilogy(1:h2.deflate(2), abs(h2.eigen(1:h2.deflate(2), 2)-Heig(2))/abs(Heig(2)), 'b');
semilogy(1:h2.deflate(3), abs(h2.eigen(1:h2.deflate(3), 3)-Heig(3))/abs(Heig(3)), 'm');
semilogy(1:h2.deflate(4), abs(h2.eigen(1:h2.deflate(4), 4)-Heig(4))/abs(Heig(4)), 'k');
xlabel('iteration');
ylabel('error');
title('relative error in positive eigenvalues');
legend('Err(\lambda_1)', 'Err(\lambda_2)', 'Err(\lambda_3)', 'Err(\lambda_4)');
print('-depsc2', 'test1_2.eps');

figure(4);
semilogy(1:h4.deflate(1), abs(h4.ev(1:h4.deflate(1), 1)-Heig(1))/abs(Heig(1)), 'r');
hold on;
semilogy(1:h4.deflate(2), abs(h4.ev(1:h4.deflate(2), 2)-Heig(2))/abs(Heig(2)), 'b');
semilogy(1:h4.deflate(3), abs(h4.ev(1:h4.deflate(3), 3)-Heig(3))/abs(Heig(3)), 'm');
semilogy(1:h4.deflate(4), abs(h4.ev(1:h4.deflate(4), 4)-Heig(4))/abs(Heig(4)), 'k');
xlabel('iteration');
ylabel('error');
title('relative error in positive eigenvalues');
legend('Err(\lambda_1)', 'Err(\lambda_2)', 'Err(\lambda_3)', 'Err(\lambda_4)');
print('-depsc2', 'test1_4.eps');

figure(5);
semilogy(1:h5.deflate(1), abs(h5.eigen(1:h5.deflate(1), 1)-Heig(1))/abs(Heig(1)), 'r');
hold on;
semilogy(1:h5.deflate(2), abs(h5.eigen(1:h5.deflate(2), 2)-Heig(2))/abs(Heig(2)), 'b');
semilogy(1:h5.deflate(3), abs(h5.eigen(1:h5.deflate(3), 3)-Heig(3))/abs(Heig(3)), 'm');
semilogy(1:h5.deflate(4), abs(h5.eigen(1:h5.deflate(4), 4)-Heig(4))/abs(Heig(4)), 'k');
xlabel('iteration');
ylabel('error');
title('relative error in positive eigenvalues');
legend('Err(\lambda_1)', 'Err(\lambda_2)', 'Err(\lambda_3)', 'Err(\lambda_4)');
print('-depsc2', 'test1_5.eps');
end
