function [iterFinal,iterPosNeg]=testQuadraticAMG(n)
% this function compares indefinite_m_scheme for m=2,3,4,5,6 and m='whole'
%(meaning: using whole history) with two AMG preconditioners applied to a 
% linearization of dimension 2*n
% of scalable QEP (lambda^2 M+lambda D+K)x=0 of dimension n
% author: Marija Miloloza Pandur, e-mail:mmiloloz@mathos.hr, 12.11.2016.

addpath('/home/mmiloloz/Alati/hsl_mi20-2.0.0/matlab/');
javaaddpath('/home/mmiloloz/Alati/hsl_mi20-2.0.0/matlab/');

%l_pos=3;l_neg=3; % number of wanted eigenpairs
k_pos=3;k_neg=3; % number of extracted eigenpairs (k_\pm >= l_\pm)
tol=1e-7;maxiter=500;
alpha = (n+1)^1;
M=speye(n);
e = ones(n,1);
K = (n+1)^2*full(spdiags([-e 2*e -e], -1:1, n, n));
K=sparse(K);
D=2*K;
%K = spdiags([-5*e 15*e -5*e], -1:1, n, n); D = spdiags([-10*e 30*e -10*e], -1:1, n, n); % definiteness interval is around ( -9.4818, -0.5278)
%% exact eigenvalues of QEP with matrices M,D,K
for j = 1:n,
    t = j*pi/(n+1);
    u(2*j-1:2*j) = roots([1, 4*(n+1)^2*(1-cos(t)),2*(n+1)^2*(1-cos(t))]);
end
u = sort(u');
%% linearization
A=blkdiag(alpha^2*M, -K);B=[sparse(n,n) alpha*M;alpha*M D];%2n x 2n pencil
A=A/(n+1)^2; B=B/(n+1)^2;
% definiteness interval is around ( -19.225842, -0.513351)
%% initial guess
%I=speye(2*n);
II=speye(n);
%Xini=full([[M,II;-II,II]*I(:,n+1:(n+k_neg)) I(:,1:k_pos)]);
Xini=full([[D(:,1:k_neg)/alpha;-II(:,1:k_neg)],[sparse(n,k_pos);II(:,1:k_pos)]]);

%% 1. TEST: mu2p and mu2n
mu2p=-0.514;
mu2n=-19.22;
T2p=A-mu2p*B;
T2n=A-mu2n*B;
%% HSL_mi20 preconditioner
hsl_mi20_startup
control2p = hsl_mi20_control;
control2p.c_fail = 2;
inform2p = hsl_mi20_setup2p(T2p,control2p);
control2n = hsl_mi20_control;
control2n.c_fail = 2;
inform2n = hsl_mi20_setup2n(T2n,control2n);
flag=[1,1,1,1,1,1];
%% testing
if flag(1),
    msch=2; 
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    tic
    [D2, X, iter2,h2] = indef_m_scheme_brzo(A, B, 'hsl_mi20_precondition2p', 'hsl_mi20_precondition2n', Xini, k_pos, k_neg, tol, maxiter,msch);
    disp([D2, abs(D2-u(n+(-k_neg+1:k_pos)))./abs(u(n+(-k_neg+1:k_pos)))]);
    t2=toc;
    err2=norm(A*X-B*X*diag(D2));
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    fprintf('#(iterations) = %d.\n\n', iter2);
    save('output2000/output.mat','iter2');
    save('output2000/output.mat','D2','-append');
    save('output2000/output.mat','h2','-append');
    save('output2000/output.mat','t2','-append');
end
if flag(2),
    msch=3; 
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    tic
    [D3, X, iter3,h3] = indef_m_scheme_brzo(A, B, 'hsl_mi20_precondition2p', 'hsl_mi20_precondition2n', Xini, k_pos, k_neg, tol, maxiter,msch);
    disp([D3, abs(D3-u(n+(-k_neg+1:k_pos)))./abs(u(n+(-k_neg+1:k_pos)))]);
    t3=toc;
    err3=norm(A*X-B*X*diag(D3));
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    fprintf('#(iterations) = %d.\n\n', iter3);
    save('output2000/output.mat','iter3');
    save('output2000/output.mat','D3','-append');
    save('output2000/output.mat','h3','-append');
    save('output2000/output.mat','t3','-append');
end
if flag(3),
    msch=4; 
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    tic
    [D4, X, iter4,h4] = indef_m_scheme_brzo(A, B, 'hsl_mi20_precondition2p', 'hsl_mi20_precondition2n', Xini, k_pos, k_neg, tol, maxiter,msch);
    disp([D4, abs(D4-u(n+(-k_neg+1:k_pos)))./abs(u(n+(-k_neg+1:k_pos)))]);
    t4=toc;
    err4=norm(A*X-B*X*diag(D4));
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    fprintf('#(iterations) = %d.\n\n', iter4);
    save('output2000/output.mat','iter4');
    save('output2000/output.mat','D4','-append');
    save('output2000/output.mat','h4','-append');
    save('output2000/output.mat','t4','-append');
end
if flag(4),
    msch=5; 
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    tic
    [D5, X, iter5,h5] = indef_m_scheme_brzo(A, B, 'hsl_mi20_precondition2p', 'hsl_mi20_precondition2n', Xini, k_pos, k_neg, tol, maxiter,msch);
    disp([D5, abs(D5-u(n+(-k_neg+1:k_pos)))./abs(u(n+(-k_neg+1:k_pos)))]);
    t5=toc;
    err4=norm(A*X-B*X*diag(D5));
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    fprintf('#(iterations) = %d.\n\n', iter5);
    save('output2000/output.mat','iter5');
    save('output2000/output.mat','D5','-append');
    save('output2000/output.mat','h5','-append');
    save('output2000/output.mat','t5','-append');
end
if flag(5),
    msch=6; 
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    tic
    [D6, X, iter6,h6] = indef_m_scheme_brzo(A, B, 'hsl_mi20_precondition2p', 'hsl_mi20_precondition2n', Xini, k_pos, k_neg, tol, maxiter,msch);
    disp([D6, abs(D6-u(n+(-k_neg+1:k_pos)))./abs(u(n+(-k_neg+1:k_pos)))]);
    t6=toc;
    err6=norm(A*X-B*X*diag(D6));
    fprintf('\n Test2: Algorithm indef_%d_scheme (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',msch,mu2p,mu2n);
    fprintf('#(iterations) = %d.\n\n', iter4);
    save('output2000/output.mat','iter6');
    save('output2000/output.mat','D6','-append');
    save('output2000/output.mat','h6','-append');
    save('output2000/output.mat','t6','-append');
end
if flag(6),
    msch='whole'; 
    fprintf('\n Test2: Algorithm indef_m_scheme with whole history (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',mu2p,mu2n);
    tic
    [Dw, X, iterw,hw] = indef_m_scheme_brzo(A, B, 'hsl_mi20_precondition2p', 'hsl_mi20_precondition2n', Xini, k_pos, k_neg, tol, maxiter,msch);
    disp([D4, abs(D4-u(n+(-k_neg+1:k_pos)))./abs(u(n+(-k_neg+1:k_pos)))]);
    tw=toc;
    errw=norm(A*X-B*X*diag(Dw));
    fprintf('\n Test2: Algorithm indef_m_scheme with whole history (two preconditioners) with T_+ \appr inv(A-%f*B), T_-=inv(A-%f*B) \n',mu2p,mu2n);
    fprintf('#(iterations) = %d.\n\n', iterw);
    save('output2000/output.mat','iterw');
    save('output2000/output.mat','Dw','-append');
    save('output2000/output.mat','hw','-append');
    save('output2000/output.mat','tw','-append');
end
if flag==[1,1,1,1,1,1]
   disp('first row:m; second row: total number of iterations')
  iterFinal=[2,3,4,5,6,1000;iter2,iter3,iter4,iter5,iter6,iterw]  
  disp('first row:m; second row: total number of iterations for B-pos; third row: total number of iterations for B-neg;')
  iterPosNeg=[2,3,4,5,6,1000;h2.step_pos(end),h3.step_pos(end),h4.step_pos(end),h5.step_pos(end),h6.step_pos(end),hw.step_pos(end);h2.step_neg(end),h3.step_neg(end),h4.step_neg(end),h5.step_neg(end),h6.step_neg(end),hw.step_neg(end)]    
end
hsl_mi20_finalize2p;
hsl_mi20_finalize2n;