function hsl_mi20_install(varargin)
%
%   hsl_mi20_install installs hsl_mi20.
%
%   VARARGIN
%   
%   HSL_MI20_INSTALL() installs hsl_mi20 and its Matlab Interface. It is 
%       assumed that the BLAS and LAPACK routines provided with the interface 
%       are used, the relevent version of g95 can be called by the command 
%       "g95" and that the default libraries are available for use. 
%       The test example is not run.
%
%   HSL_MI20_INSTALL(TEST) installs hsl_mi20 and its Matlab Interface. It is 
%       assumed that the BLAS and LAPACK routines provided with the interface 
%       are used, the relevent version of g95 can be called by the command 
%       "g95" and that the default libraries are available for use. 
%       If TEST <= 0, the test example is not run; 
%       if TEST > 0, the test example is run.
%
%   HSL_MI20_INSTALL(TEST,BLAS) installs hsl_mi20 and its Matlab Interface 
%       and optionally runs the test example. It is assumed that the relevant 
%       version of g95 can be called by the command "g95" and the default 
%       compiler libraries are available for use.
%       If BLAS has the value 'matlab' then the matlab provided BLAS/LAPACK are
%         used. If the installed BLAS/LAPACK have any bugs, then this may not
%         be robust and alternate BLAS should be used.
%       Otherwise BLAS should provide a list of files (eg 'hsl_mi20d_blas.f') or
%         link flags (eg '-L/path/to/blas -lblas') that provide the required
%         BLAS/LAPACK routines.
%
%   HSL_MI20_INSTALL(TEST,BLAS,COMPILER) installs hsl_mi20 and its Matlab 
%       Interface. COMPILER is a string and it must contain the path for the 
%       relevent g95 compiler. It is assumed that the default compiler libraries
%       are available for use.
%
%   HSL_MI20_INSTALL(TEST,BLAS,COMPILER,LIBF95) installs hsl_mi20 and its 
%       Matlab Interface. LIBF95 is a string and must contain the path where the 
%       desired version of libf95.a is located, for example, 
%       '/usr/local/g95-install/lib/gcc-lib/i686-pc-linux-gnu/4.0.3'. 
%       Default gcc library is used.
%
%   HSL_MI20_INSTALL(TEST,BLAS,COMPILER,LIBF95,LIBGCC) attempts to install 
%       hsl_mi20 and its Matlab Interface. LIBGCC is a string and must contain 
%       the path where the desired version of libgcc.a is located, for example,
%       '/usr/lib/gcc/i386-redhat-linux/3.4.6'.
%
%   See also  PATHTOOL, JAVAADDPATH, ADDPATH.
%   Also see DOC STARTUP.
% 

% srcdir is the directory containing the Fortran source code for the package
srcdir = '..'

nin  = nargin;
if (nin > 5),
   error('Too many arguments');
end;

v = getversion ;

%-------------------------------------------------------------------------------
% Check that Matlab version is recent enough
%-------------------------------------------------------------------------------

if (v <= 7.3)
	error('Matlab version too old - require version 7.4 (2007a) or newer');
end

%-------------------------------------------------------------------------------
% Check that we are on a unix/linux machine
%-------------------------------------------------------------------------------
if (~(isunix))
	error('Not a unix/linux system');
end

if (nin>=2)
    blas = varargin{2};
else
    blas = 'matlab';
end

if (nin>=3)
    g95str = strtrim(varargin{3});
else
    g95str = 'g95';
end


if (nin>=4)
	str = strtrim(varargin{4});
	if (str(end)=='/')
		str = strcat(str,'libf95.a ');
	else
		str = strcat(str,'/libf95.a ');
   end
	if (exist(str,'file'))
	   libf95    = strcat(' -L ',str,' -lf95 ');
	else
	   mesg = strcat('The library libf95.a is not in the directory ',...
		varargin{4},' given in input');
	   error(mesg)
	end
else
	libf95    = '';
end

if (nin>=5)
	str = strtrim(varargin{5});
	if (str(end)=='/')
		str = strcat(strtrim(varargin{5}),'libgcc.a');
	else
		str = strcat(strtrim(varargin{5}),'/libgcc.a');
	end
	if (exist(str,'file'))
	   libf95    = strcat(libf95,' -L ',varargin{4},' ');
	else
	   mesg = strcat('The library libgcc.a is not in the directory ',...
		varargin{4},' given in input');
	   error(mesg)
	end
end

d = '';
is64 = (~isempty(strfind(computer,'64')));
if (is64)
  % 64-bit MATLAB
  fcstr = strcat(' FC='' ',g95str,...
	' '' FFLAGS=''-fexceptions -fPIC -fno-omit-frame-pointer -i8'' ');
  g95str = strcat(g95str,' ',' -O3 -Wno=155 ');
  d = ' -fPIC -i8  -fno-second-underscore';
  g95str = strcat(g95str,d);
else
  fcstr = strcat(' FC='' ',g95str,...
	' '' FFLAGS=''-fexceptions -fPIC -fno-omit-frame-pointer '' ');
  g95str = strcat(g95str,' ',' -O3 -Wno=155 ');
end

kk = do_unix_cmd('rm  *.o *.mod *.mexglx *.mexa64');
%-------------------------------------------------------------------------------
% BLAS option
%-------------------------------------------------------------------------------
if (strcmp(blas,'matlab')==1)
	if (v <= 7.5)
  		lib = ' -llapack -lblas';
	else
  		lib = ' -lmwlapack -lmwblas';
	end
else
  	lib = [' ', blas];
end


str = [g95str, ' -o ddeps.o -c ', srcdir ,'/ddeps.f'];
kk = do_unix_cmd(str);
if (kk~=0)
   error('Compile failed.');
end
str = [g95str, ' -o ddeps90.o -c ', srcdir ,'/ddeps90.f90'];
kk = do_unix_cmd(str);
if (kk~=0)
   error('Compile failed.');
end
str = [g95str, ' -o hsl_mi20d.o -c ', srcdir ,'/hsl_mi20d.f90'];
kk = do_unix_cmd(str);
if (kk~=0)
   error('Compile failed.');
end
mexpath   = matlabroot;
MEXpath   = strcat('MEXpath      =  ',mexpath);
str = strcat(g95str, ' -o hsl_matlab.o -c hsl_matlab.F90 -I ',mexpath,'/extern/include');
kk = do_unix_cmd(str);
if (kk~=0)
   error('Compile failed.');
end
str = strcat(mexpath,'/bin/mex ',fcstr, '  -g -largeArrayDims',lib,' -I ',...
	mexpath,'/extern/include hsl_mi20.F90 ',libf95,...
	' -output hsl_mi20 ', ...
	' ddeps.o ddeps90.o hsl_matlab.o hsl_mi20d.o');
kk = do_unix_cmd(str);
if (kk~=0)
   error('Compile failed.');
end

%mexpath   = matlabroot;
%MEXpath   = strcat('MEXpath      = \t ',mexpath);
%fid       = fopen('Makefile.in','w');
%fprintf(fid,MEXpath);
%fprintf(fid,'\n');
%unix('make all');

currentpath = which (mfilename); 
sep = find (currentpath == filesep); 
currentpath = currentpath (1:sep(end));  

fprintf ('Temporarily adding %s to your MATLAB path and JAVA path.\n', currentpath) ;
fprintf ('Do this permanently via pathtool.  Next, edit the file:\n') ;
fprintf ('%s file.\n', which ('classpath.txt')) ;
fprintf ('and add the line:\n') ;
fprintf ('%s\n', currentpath) ;
fprintf ('to the end of that file (which defines your JAVA class path).\n') ;

addpath (currentpath) ;
javaaddpath (currentpath) ;

fprintf ('\nAlternatively, add these two lines to your startup.m file:\n\n') ;
fprintf ('addpath (''%s'') ;\n', currentpath) ;
fprintf ('javaaddpath (''%s'') ;\n', currentpath) ;
fprintf ('\nSee also pathtool, javaaddpath, addpath, and "doc startup".\n\n') ;

if (nin >= 1 & varargin{1}>0),	 
hsl_mi20_test;
end;
clear nin v currentpath sep mexpath MEXpath

%-------------------------------------------------------------------------------

function v = getversion
%GETVERSION determine the MATLAB version, and return it as a double.
v = sscanf (version, '%d.%d.%d') ;
v = 10.^(0:-1:-(length(v)-1)) * v ;

%-------------------------------------------------------------------------------

function kk = do_unix_cmd(s)
% DO_UNIX_CMD executes the unix command that is supplied within the string s.
fprintf('%s\n',s);
kk = unix(s);
